# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chain',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='ChainElement',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('joiner', models.CharField(blank=True, max_length=3, choices=[('AND', 'and'), ('OR', 'or')])),
                ('negated', models.BooleanField(default=False)),
                ('order', models.PositiveSmallIntegerField()),
                ('chain', models.ForeignKey(to='django_condition_chain.Chain')),
            ],
        ),
        migrations.CreateModel(
            name='Condition',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=64)),
                ('module', models.CharField(help_text='Module in which the condition function resides', max_length=128)),
                ('function', models.CharField(help_text='The function which returns True or False to determine the result of this condition', max_length=64)),
            ],
        ),
        migrations.AddField(
            model_name='chainelement',
            name='condition',
            field=models.ForeignKey(to='django_condition_chain.Condition'),
        ),
        migrations.AlterUniqueTogether(
            name='chainelement',
            unique_together=set([('chain', 'order')]),
        ),
    ]



from __future__ import unicode_literals
from importlib import import_module

from django.utils.encoding import python_2_unicode_compatible
from django.db import models


@python_2_unicode_compatible
class Condition(models.Model):
    """
    Stores the details of a Python function which will be used to determine this condition's
    truthiness.
    """
    name = models.CharField(max_length=64)
    module = models.CharField(
        max_length=128, help_text="Module in which the condition function resides")
    function = models.CharField(
        max_length=64,
        help_text=("The function which returns True or False to determine the result of this "
                   "condition"))

    def __str__(self):
        return self.name

    def call(self, *args, **kwargs):
        """
        Imports and calls the condition function with the provided arguments and returns its
        response.
        """
        return getattr(import_module(self.module), self.function)(*args, **kwargs)


class Chain(models.Model):
    name = models.CharField(max_length=128)

    def call(self, *args, **kwargs):
        """
        Runs each condition and evaluates whether the entire condition chain has succeeded. Passes
        through any arguments provided to each Condition.
        """
        # Artificially set the first result to True to make our lives easier
        # when joining the rest of the conditions together.
        expression = "True"
        for element in self.elements_queryset:
            expression += " %s " % element.joiner.lower()
            expression += "%s" % bool(element.condition.call(*args, **kwargs))
        return eval(expression)

    def __iter__(self):
        return iter(self.elements_queryset)

    def __len__(self):
        return self.elements_queryset.count()

    def __getitem__(self, key):
        return self.elements_queryset[key]

    def __reversed__(self):
        return self.elements_queryset.order_by("-order")

    @property
    def elements_queryset(self):
        return ChainElement.objects.filter(chain=self).order_by("order")

    # @transaction.atomic
    # def append(self, element):
    #     """
    #     Update the element's chain property to be this Chain, and set its order
    #     to be the last in the current list of elements.
    #     """
    #     next_order = reversed(self).only("order")[0].order + 1
    #     element.chain = self
    #     element.order = 0
    #     element.save()
    #     ChainElement.objects.filter(pk=element.pk).update(order=next_order)


@python_2_unicode_compatible
class ChainElement(models.Model):
    links = (
        (u'and', u'and'),
        (u'or', u'or')
    )
    chain = models.ForeignKey(Chain)
    joiner = models.CharField(choices=links, max_length=3, default="and")
    negated = models.BooleanField(default=False)
    condition = models.ForeignKey(Condition)
    order = models.PositiveSmallIntegerField()

    class Meta:
        unique_together = (
            ("chain", "order"),
        )

    def __str__(self):
        isnt = "NOT " if self.negated else ""
        joiner = self.joiner if self.joiner else "IF"
        return "%s %s%s" % (joiner, isnt, self.condition)



#  -*- coding: utf-8 -*-
"""
Setuptools script for the Django Condition Chain app.
"""

import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


def required(fname):
    return open(os.path.join(os.path.dirname(__file__),
                             fname)).read().split('\n')


config = {
    "name": "django-condition-chain",
    "version": "0.0.3",
    "namespace_packages": [],
    "packages": find_packages(exclude=["*.tests",
                                       "*.tests.*",
                                       "tests.*",
                                       "tests",
                                       "*.ez_setup",
                                       "*.ez_setup.*",
                                       "ez_setup.*",
                                       "ez_setup",
                                       "*.examples",
                                       "*.examples.*",
                                       "examples.*",
                                       "examples", ]),
    "include_package_data": True,
    "package_data": {"": ["*.js"], },
    "scripts": [],
    "entry_points": {},
    "install_requires": [required('requirements.txt')],
    "tests_require": [required('test-requirements.txt')],
    "test_suite": 'nose.collector',
    "zip_safe": False,
    # Metadata for upload to PyPI
    "author": 'Ellis Percival',
    "author_email": "ellis@0x07.co.uk",
    "description": "Django Condition Chain app",
    "classifiers": ["Programming Language :: Python", ],
    "license": "",
    "keywords": "",
    "url": "",
}

setup(**config)
